<?php

namespace FiloBlu\Kpi\Model;
/**
 * Class Createviews
 * @package FiloBlu\Kpi\Model
 */
class Createviews extends Core
{
    /**
     * @return bool
     */
    public function createViews()
    {
        $this->createCustomersBiView();
        $this->createProductsBiView();
        return true;
    }

    /**
     * @return bool
     */
    public function createCustomersBiView()
    {
        $queryCustomer = $this->getCustomerCollection()->getSelect()->__toString();
        $queryFinalCustomer = 'CREATE OR REPLACE VIEW `customer_bi` AS ' . str_replace('INNER', 'LEFT', $queryCustomer);
        $this->getConnection()->query($queryFinalCustomer);
        return true;
    }

    /**
     * @return bool
     */
    public function createProductsBiView()
    {

        // This will return one array with all the storeview ID
        $affected_stores = $this->getDefaultStoreIds();

        $final_sql = [];

        $attributes = $this->getAttributesFromKpiConfig();

        // Each cycle of the affected stores
        // I need to keep in mind the products that I have
        // and exclude them the next cycle
        // This to prevent duplicated SKUs inside the view
        // TODO: Find a way to implement a good GROUP BY
        $products_already_inside_the_view = array();

        foreach ($affected_stores as $store_id_from_config) {
            $productCollection = $this->getProductsCollection();
            $productCollection->addStoreFilter($this->_storeManagerInterface->getStore($store_id_from_config));
            $productCollection->addAttributeToFilter('type_id', 'simple');

            foreach ($attributes as $attribute_data) {
                $productCollection->addAttributeToSelect($attribute_data['catalog_product_attribute'], 'left');
            }

            if(count($products_already_inside_the_view) > 0){
                $productCollection->addFieldToFilter('entity_id',array('nin'=>$products_already_inside_the_view));
            }
            if (count($affected_stores) > 1 && $productCollection->getSize() > 0){
                $products_already_inside_the_view = array_unique(array_merge($products_already_inside_the_view,$productCollection->getAllIds()));
            }

            $queryProduct = $productCollection->getSelect()->__toString();

            //Fixing column names for the view
            foreach ($attributes as $attribute_data) {
                $string_to_change = " AS `{$attribute_data['catalog_product_attribute']}`";
                $change_in = " AS `{$attribute_data['column_name']}`";
                $queryProduct = str_replace($string_to_change, $change_in, $queryProduct);
            }

            $final_sql[] = $queryProduct;

            $productCollection->clear();
        }

        $final_sql = implode("\n\n UNION ALL \n\n", $final_sql);

        $create_view = "CREATE OR REPLACE VIEW `product_bi` AS SELECT DISTINCT * FROM ( {$final_sql} ) AS TMP";

        $this->getConnection()->query($create_view);

        return true;
    }

    /**
     * This function will read all the attributes from the kpi/table_configuration/fields config
     * And return one array with them
     */
    public function getAttributesFromKpiConfig()
    {
        $attributes = $this->_scopeConfig->getValue('kpi/table_configuration/fields');

        if (!$attributes) {
            return false;
        }

        $logger = $this->_helper->getLogger('filoblu_kpi.log');

        $attributes = $this->_serializer->unserialize($attributes);

        $attributes_array = [];
        $attributesToCheck = [];
        $connection = $this->getConnection();

        foreach ($attributes as $key => $data) {
            // Fallback if missing column_name from backend config
            if (!$data['column_name']) {
                $data['column_name'] = $data['catalog_product_attribute'];
            }
            $attributesToCheck[] = "'{$data['catalog_product_attribute']}'";
            $attributes_array[] = $data;
        }


        if(empty($attributesToCheck))
        {
            return false;
        }

        // TODO: use better syntax
        $eavTable = $connection->getTableName('eav_attribute');
        $result = $connection->fetchAll("SELECT `attribute_code` FROM `{$eavTable}` WHERE `entity_type_id` = 4 AND `attribute_code` IN (" . join(',', $attributesToCheck) . ")", \Zend_Db::FETCH_ASSOC);

        $existingAttributes = [];
        foreach ($result as $a) {
            $existingAttributes[] = $a['attribute_code'];
        }

        $output = [];

        foreach ($attributes_array as $item) {
            $attributeCode = $item['catalog_product_attribute'];
            if (!in_array($attributeCode, $existingAttributes)) {
                $logger->warn("Attribute with code '{$attributeCode}' does not exists. Check your mapping");
                continue;
            }
            $output[] = $item;
        }

        if(empty($output))
        {
            return false;
        }

        return $output;
    }

}